/*
* ==============================================================================
*  Name        : ShoutcastAdapter.cpp
*  Part of     : Shoutcast Engine
*  Description : This class implements the shoutcast adapter interface.
*  Version     : 1
*
*  Copyright (c) 2006, Nokia Corporation All rights reserved. Redistribution
*  and use in source and binary forms, with or without modification, are
*  permitted provided that the following conditions are met: Redistributions
*  of source code must retain the above copyright notice, this list of
*  conditions and the following disclaimer. Redistributions in binary form
*  must reproduce the above copyright notice, this list of conditions and the
*  following disclaimer in the documentation and/or other materials provided
*  with the distribution. Neither the name of the Nokia Corporation nor the
*  names of its contributors may be used to endorse or promote products
*  derived from this software without specific prior written permission. THIS
*  SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
*  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
*  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
*  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
*  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*  POSSIBILITY OF SUCH DAMAGE.
* ==============================================================================
*/

// INCLUDE FILES
#include <eikmenup.h>
#include <eikapp.h>
#include <ImplementationProxy.h>
#include <MAdapterObserver.h>
#include <S60InternetRadio.hrh>
#include <ckninfo.h>
//#include <S60InternetRadio.rsg>
#include <eikenv.h>
#include "ShoutcastDefs.h"
#include "ShoutcastErrors.h"
#include "ShoutcastUIDs.hrh"
#include "ShoutcastAdapter.h"
#include "ShoutcastStream.h"
#include "scmetadata.h"

// Identifying string for this audio utility
_LIT(KShoutcastAdapter, "Internet Radio");

const TInt KDefaultVolumeLevel = 2;
const TInt KMaxFileSize = 65535;
const TInt KLineFeed = 0x0A;
const TInt KCarriageReturn = 0x0D;

_LIT(KConnecting, "Connecting");
_LIT(KError, "Error");


// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// CShoutcastAdapter::CShoutcastAdapter
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//
CShoutcastAdapter::CShoutcastAdapter(
	MAdapterObserver *aObserver )
	: CPluginAdapter(aObserver)
    {
	iState = ENotReady;
	iPrimed = EFalse;
    }

// -----------------------------------------------------------------------------
// CShoutcastAdapter::NewL
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CShoutcastAdapter* CShoutcastAdapter::NewL(
	MAdapterObserver *aObserver )
    {
    CShoutcastAdapter* self = new (ELeave) CShoutcastAdapter(aObserver);
    CleanupStack::PushL(self);
    self->ConstructL();
	CleanupStack::Pop(self);
    return self;
    }

// -----------------------------------------------------------------------------
// CShoutcastAdapter::ConstructL
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::ConstructL()
	{
	LOG_START;
	TParse parse;

	_LIT(dir, "\\Shoutcast\\");
	_LIT(wildName,"*.pls");

    FindPlayListsL(dir, wildName);
	}


// -----------------------------------------------------------------------------
// CShoutcastAdapter::FindPlayLists
// Open the requested URL and create the stream
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::FindPlayListsL(
	const TDesC& aDir,
	const TDesC& aWildname )
	{
	RFs fs;
	RFile file;
	TBuf<50> text;

	User::LeaveIfError(fs.Connect());
	CleanupClosePushL(fs);

	text.Copy(_L("Building Playlist"));
    CEikonEnv::Static()->BusyMsgL(text);

	TFindFile file_finder(fs);
    CDir* file_list;
    TInt err = file_finder.FindWildByDir(aWildname, aDir, file_list);

    while ( err == KErrNone )
        {
        TInt i;
        for (i=0; i<file_list->Count(); i++)
            {
            TParse fullentry;
            fullentry.Set((*file_list)[i].iName,& file_finder.File(),NULL);
            // It's ok if the parse fail leave on this file, we'll try to parse the next one.
            TRAPD(error, ParsePlaylistFileL(fs, fullentry.FullName()));
            }
        delete file_list;
        err=file_finder.FindWild(file_list);
        }

	CleanupStack::Pop();
	fs.Close();

    CEikonEnv::Static()->BusyMsgCancel();

	text.Copy(_L("Select a station to listen."));
    CEikonEnv::Static()->InfoMsg(text);
	}



// -----------------------------------------------------------------------------
// CShoutcastAdapter::OpenUrlL
// Open the requested URL and create the stream
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::OpenUrlL(
	TInt aUrl )
	{
	LOG("OpenUrlL");

	iStreamSource = CShoutcastStream::NewL(*iURLs[aUrl], *this);
	iStreamSource->SetVolume(KDefaultVolumeLevel);
	iNrMetadata = iStreamSource->GetNumberOfMetaDataEntries();

	iState=EReadyToPlay;
	iPrimed=EFalse;
	iObserver->ResetMetadata();

	LOG("OpenUrlL OK");
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::CloseUrl
// Closes the currently opened URL and delete the stream
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::CloseUrl()
	{
	LOG("CloseUrl");
	delete iStreamSource;
	iStreamSource = NULL;
	iState = ENotReady;
	LOG("CloseUrl OK");
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::~CShoutcastAdapter
// -----------------------------------------------------------------------------
//
CShoutcastAdapter::~CShoutcastAdapter()
	{
	LOG("Destructor");
	CloseUrl();
	delete iWaitDialog;
	iWaitDialog = NULL;
    iURLs.ResetAndDestroy();
	LOG("Destructor OK");
	LOG_END;
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::GetMenuText
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::GetMenuText(
	TDes &aName )
	{
	LOG("GetMenuText")
	aName.Copy(KShoutcastAdapter);
	LOG("GetMenuText OK")
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::GetMenuCommand
// This menu command number is used on the UI level. When the user selects a
// utility on the menu, this command number is used to find this adapter.
// The usage is like the hard-coded command values in the menu resource (rss).
// -----------------------------------------------------------------------------
//
TInt CShoutcastAdapter::GetMenuCommand()
	{
	return 0x2001; //Should be a defined constant
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::AddExtraMenu
// Add each item in the playlist as an entry to the "Stations" menu.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::AddExtraMenu(
	CEikMenuPane* aMenuPane )
	{
	LOG("AddExtraMenu")

	CEikMenuPaneItem::SData sd;

	//LOG1("URL count %d", iURLs.Count());

	for ( TInt i = 0; i< iURLs.Count(); i+=2 )
		{
        sd.iCommandId = 0x2010 + (TInt)(i/2);
		sd.iText.Copy(iURLs[i+1]->Left(30));
		sd.iFlags = 0;
        sd.iCascadeId = 0;
		TRAPD(error, aMenuPane->AddMenuItemL(sd));
		}

	LOG("AddExtraMenu OK")
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::HandleCommandL
// When a station is selected from the UI, it is translated into a command which
// is mapped to a URL. The command index is used to lookup the URL.
// -----------------------------------------------------------------------------
//
TBool CShoutcastAdapter::HandleCommandL(
	TInt aCommand )
	{

	LOG("HandleCommandL")

	for ( TInt i=0; i < iURLs.Count(); i+=2 )
		if ( aCommand == 0x2010 + (TInt)(i/2) )
			{
			if ( iState == EPlaying )
				{
				StopL();
				}
			CloseUrl();
			OpenUrlL(i);//this method can leave
			PlayL();
			return ETrue;
			}

	LOG("HandleCommandL OK")

	return EFalse;
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::UpdateMenuL
// The menu pane is updated to show only options available in the current state
// of the adapter.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::UpdateMenuL(CEikMenuPane* aMenuPane)
	{
	LOG("UpdateMenuL")

    aMenuPane->SetItemDimmed(ES60InternetRadioCmdPlay,   ETrue);
    aMenuPane->SetItemDimmed(ES60InternetRadioCmdStop,   ETrue);
    aMenuPane->SetItemDimmed(ES60InternetRadioCmdChange, ETrue);
	aMenuPane->SetItemDimmed(ES60InternetRadioCmdFileSelect, ETrue);

    switch ( iState )
        {
    	case ENotReady:
        	aMenuPane->SetItemDimmed(ES60InternetRadioCmdChange, EFalse);
        	break;
    	case EReadyToPlay:
        	//aMenuPane->SetItemDimmed(ES60InternetRadioCmdPlay, EFalse);
        	aMenuPane->SetItemDimmed(ES60InternetRadioCmdChange, EFalse);
        	break;
    	case EPlaying:
        	aMenuPane->SetItemDimmed(ES60InternetRadioCmdStop, EFalse);
        	break;
    	default:
        	User::Panic(KShoutcastAdapter, 0);
        	break;
        }

	//the extra menu is available to shoutcast
	aMenuPane->SetItemDimmed(ES60InternetRadioCmdExtra,EFalse);
	aMenuPane->SetItemTextL(ES60InternetRadioCmdExtra,_L("Stations"));

	LOG("UpdateMenuL OK")

	}

// -----------------------------------------------------------------------------
// CShoutcastStream::PlayL
// Starts the stream playback
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::PlayL()
	{

	LOG("CShoutcastAdapter::PlayL")
	TInt err;

	if ( iState != EReadyToPlay )
		{
		User::Leave(KErrNotReady);
		}

	if ( !iPrimed )
		{
		// The source is not primed, so prime it first
		TRAP(err, iStreamSource->PrimeL());
		if ( err )
			{
			HandleError(err, EOpPrime);
			iStreamSource->Stop();
			CloseUrl();
			return;
			}
		//the actual play command is given when we are connected
		}
	else
		{
		//the shoutcast stream was only paused
		//issue PLay command
		TRAP(err, iStreamSource->PlayL());
		if ( err )
			{
			HandleError(err, EOpPlay);
			iStreamSource->Stop();
			return;
			}
		iState = EPlaying;
		//update metadata
		CMetaDataEntry *mdEntry = NULL;
		TRAP(err, mdEntry = iStreamSource->GetMetaDataEntryL(3));
		if ( !err )
			{
			if ( iObserver )
				{
				iObserver->SetMetadata(3, mdEntry->Value(), ETrue);
				}
			delete mdEntry;
			mdEntry = NULL;
			}
		}
	LOG("CShoutcastAdapter::PlayL OK")

	}


// -----------------------------------------------------------------------------
// CShoutcastAdapter::PauseL
// This implementation pauses the playback but doesn't stop receiving
// streaming data thus allowing the playback to resume quickly.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::PauseL()
	{
	LOG("CShoutcastAdapter::PauseL")
    if ( iState == EPlaying )
		{
		iStreamSource->Pause();
		iState = EReadyToPlay;
		TBufC<10> buf=_L("Paused");
		DeleteLabels(EFalse,&buf);
		}
	LOG("CShoutcastAdapter::PauseL OK")
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::RecordL
// This is not supported.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::RecordL()
    {
	User::Leave(KErrNotSupported);
    }

// -----------------------------------------------------------------------------
// CShoutcastAdapter::StopL
// Stops the playback and the URL is also closed.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::StopL()
	{
	LOG("CShoutcastAdapter::StopL")
	if ( iState == EPlaying )
		{
		iStreamSource->Stop();
		iState = EReadyToPlay;
		iPrimed = EFalse;
		TBufC<10> buf=_L("Stopped");
		DeleteLabels(EFalse,&buf);
		}
	LOG("CShoutcastAdapter::StopL OK")
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::Identify
// Returned the identity of the adapter.
// -----------------------------------------------------------------------------
//
const TDesC& CShoutcastAdapter::Identify() const
    {
    return KShoutcastAdapter;
    }


// -----------------------------------------------------------------------------
// CShoutcastAdapter::SupportMetadata
//
// -----------------------------------------------------------------------------
//
TInt CShoutcastAdapter::SupportMetadata() const
	{
	return iNrMetadata;
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::TogglePlayPause
// Toggles play and stop.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::TogglePlayPause()
	{
	if ( iState == EReadyToPlay )
		{
		PlayL();
		}
	else if ( iState == EPlaying )
		{
		PauseL();
		}
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::HandleEvent
// Handle events from the shoutcast stream.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::HandleEvent(
	TUid aEvent,
	TInt aError)
	{
	TInt r;
	LOG2("HandleEvent: We got an Event: %x %d", aEvent, aError);

	// Metadata update event
	if ( aEvent == TUid::Uid(KShoutcastStreamUid) && aError > 0x0100 && aError < 0x0200 )
		{
		// We have a metadata event
		TInt i,f,e;
		CMetaDataEntry *mdEntry = NULL;

		f = aError&0xFF; // bitmap of metadata indicators

		// Check which metadata got updated
		for( i=0; i < iNrMetadata; i++)
			{
			e = f&0x01;
			f = f>>1;
			if( e )
				{
				// Updated metadata found
				LOG1("HandleEvent - Update Metadata %d", i);

				TRAPD(err, mdEntry = iStreamSource->GetMetaDataEntryL(i));
				if ( !err )
					{
					if( iObserver )
						{
						iObserver->SetMetadata(i, mdEntry->Value(), f==0);
						}
					delete mdEntry;
					mdEntry = NULL;
					}
				}
			}

		}

	if ( aEvent == TUid::Uid(KShoutcastStreamUid) &&
	   ( aError == KShoutcastEvent_Connecting || aError==KShoutcastEvent_Reconnecting) )
		{
		//create the text for the waiting note
		TBuf<256> text;
		CMetaDataEntry *mdEntry=NULL;
		if(	aError	==	KShoutcastEvent_Connecting	)
			{
			text.Copy(KMsgShoutcast_Connecting);
			}
		else
			{
			text.Copy(KMsgShoutcast_Reconnecting);
			}
		TRAP(r, mdEntry = iStreamSource->GetMetaDataEntryL(KShoutcastMetaData_URL));
		if( r == KErrNone )
			{
			text.Append(mdEntry->Value());
			delete mdEntry;
			}

		// pop-up waiting dialog
        delete iWaitDialog;
        iWaitDialog = NULL;
        iWaitDialog = CCknCancelDialog::NewL(&iWaitDialog, this, KConnecting, text);
		iWaitDialog->RunDlgLD();
		}

	if ( aEvent == TUid::Uid(KShoutcastStreamUid) && aError == KShoutcastEvent_Connected )
		{
		LOG("HandleEvent - Connected");

		// Dismiss the waiting dialog
		if ( iWaitDialog )
			{
            iWaitDialog->Release();
			delete iWaitDialog;
			iWaitDialog = NULL;
			}

		iPrimed = ETrue;
		//issue PLay command
		TRAPD(err, iStreamSource->PlayL());
		if ( err )
			{
			HandleError(err, EOpPlay);
			iStreamSource->Stop();
			return;
			}
		iState = EPlaying;
		}

	if ( aEvent == TUid::Uid(KShoutcastStreamUid) && aError == KShoutcastEvent_Disconnected)
		{
		//we got disconnected somehow!
		iState = EReadyToPlay;
		iPrimed = EFalse;
		DeleteLabels(ETrue);
		//try to reconnect
		PlayL(); // PlayL doesn't leave
		}

	if( aEvent == TUid::Uid(KShoutcastStreamUid) && aError==KShoutcastEvent_BufferEmpty )
		{
		//Buffer is empty! write buffering
		TBufC<10> buf=_L("Buffering");
		DeleteLabels(EFalse,&buf);
		}

	if ( aEvent == TUid::Uid(KShoutcastStreamUid) && aError == KShoutcastEvent_BufferingComplete )
		{
		//Buffer is full again. Write song title
		CMetaDataEntry *mdEntry = NULL;

		TRAPD(err, mdEntry=iStreamSource->GetMetaDataEntryL(3));
		if ( !err )
			{
			if ( iObserver )
				{
				iObserver->SetMetadata(3,mdEntry->Value(),ETrue);
				delete mdEntry;
				mdEntry = NULL;
				}
			}

		}

	if ( aEvent == TUid::Uid(KShoutcastStreamUid) && aError < 0)
		{
		//this is an error connecting
		HandleError(aError,EOpPrime);
		//there are no labels to delete
		}
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::CancelOperation
// Called when the waiting for connection dialog is dismissed.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::CancelOperation()
	{
    StopL();
    CloseUrl();
	}


// -----------------------------------------------------------------------------
// CShoutcastAdapter::HandleError
// Handle errors from shoutcast stream
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::HandleError(
	TInt aError,
	TOp aOperation )
	{
	TBuf<256> text;
	TBuf<15>  textErr;
	TInt globalErr = aError%100;
	TInt shoutcastErr = aError-globalErr;
	LOG2("HandleError: ShoutcastErr=%d GlobalErr=%d",shoutcastErr,globalErr);

	// dismiss the waiting dialog
	if ( iWaitDialog )
		{
        iWaitDialog->Release();
		delete iWaitDialog;
		iWaitDialog = NULL;
		}

	//first, we need to build the error text
	switch ( aOperation )
		{
		case EOpOpen:
			text.Copy(KErrMsgShoutcast_Controller);
			ASSERT(shoutcastErr == 0);
			break;
		case EOpAddDataSink:
			text.Copy(KErrMsgShoutcast_AddDataSink);
			ASSERT(shoutcastErr == 0);
			break;
		case EOpAddDataSource:
			ASSERT(shoutcastErr == 0);
			switch( globalErr )
				{
				case KErrNotFound:
					text.Copy(KErrMsgShoutcast_AddDataSource_PlaylistNotFound);
					break;
				case KErrPathNotFound:
					text.Copy(KErrMsgShoutcast_AddDataSource_URLNotFound);
					break;
				default:
					ASSERT(0);
				}
			break;
		case EOpPrime:
			text.Copy(KErrMsgShoutcast_Prime);
			break;
		case EOpPlay:
			text.Copy(KErrMsgShoutcast_Play);
		break;
		default:
			ASSERT(0);
		}

	switch ( shoutcastErr )
		{
		case KErrShoutcast_ReadFromFile:
			text.Append(KErrMsgShoutcast_ReadFromFile);
			break;
		case KErrShoutcast_ConnectToSockServer:
			text.Append(KErrMsgShoutcast_ConnectToSockServer);
			break;
		case KErrShoutcast_OpenSocket:
			text.Append(KErrMsgShoutcast_OpenSocket);
			break;
		case KErrShoutcast_Connecting2Server:
			text.Append(KErrMsgShoutcast_Connecting2Server);
			break;
		case KErrShoutcast_SendingHTTPRequest:
			text.Append(KErrMsgShoutcast_SendingHTTPRequest);
			break;
		case KErrShoutcast_ReceivingHTTPResponse:
			text.Append(KErrMsgShoutcast_ReceivingHTTPResponse);
			break;
		case KErrShoutcast_NoHTTPServer:
			text.Append(KErrMsgShoutcast_NoHTTPServer);
			break;
		case KErrShoutcast_BadIcyMetadataFormat:
			text.Append(KErrMsgShoutcast_BadIcyMetadataFormat);
			break;
		case KErrShoutcast_NoShoutcastServer:
			text.Append(KErrMsgShoutcast_NoShoutcastServer);
			break;
		case KErrShoutcast_FormatNotSupported:
			text.Append(KErrMsgShoutcast_FormatNotSupported);
			break;
		case KErrNone:
			break;
		default:
			ASSERT(0);
		}

	if ( globalErr != KErrNone )
		{
		textErr.Format(_L(" Err=%d"),globalErr);
		text.Append(textErr);
		}


	TRAPD(error, 
		{
		//build note

		LOG1("HandleError note: %s",text.PtrZ());
        CCknInfoDialog::RunDlgLD(KError, text);
		});


	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::SetVolume
// Sets the volume level
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::SetVolume(
	TInt aVolume )
	{
	LOG1("CShoutcastAdapter::SetVolume %d", aVolume);
	if ( iStreamSource )
		iStreamSource->SetVolume((TInt)(aVolume/iVolFactor));
	LOG("CShoutcastAdapter::SetVolume OK");
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::Volume
// Get the current volume level
// -----------------------------------------------------------------------------
//
TInt CShoutcastAdapter::Volume()
	{
	LOG("CShoutcastAdapter::GetVolume");
	TInt volume = KDefaultVolumeLevel;

	if ( iStreamSource )
		{
		if( !iVolFactor )
			{
			TInt maxVol = iStreamSource->MaxVolume();
			if ( maxVol != 0)
				iVolFactor=10.0/maxVol;
			}
		volume = iStreamSource->Volume();
		volume = (TInt)(iVolFactor*volume);
		};
	LOG1("CShoutcastAdapter::GetVolume: %d", volume);
	return volume;
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::DeleteLabels
// All the labels are cleared if the playing has stopped.
// Only server, genre, technical data are cleared if the playing is paused.
// The price and bytes labels always remains.
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::DeleteLabels(
	TBool aAll,
	TDesC *buf )
	{
	LOG("DeleteLabels");

	if ( !iObserver )
		{
		return;
		}

	TBufC<10> bufe=_L("");

	if ( !buf )
		{
		buf = &bufe;
		}

	iObserver->SetMetadata(3,*buf,!aAll);
	if (aAll)
		{
		iObserver->SetMetadata(0,bufe,EFalse);
		iObserver->SetMetadata(1,bufe,EFalse);
		iObserver->SetMetadata(2,bufe,ETrue);
		}

	LOG("DeleteLabels OK");
	}

// -----------------------------------------------------------------------------
// CShoutcastAdapter::ParsePlaylistFileL
// -----------------------------------------------------------------------------
//
void CShoutcastAdapter::ParsePlaylistFileL(
	RFs aFs,
	const TFileName &aFilename )
	{
	RFile file;

	TBool done = EFalse;
	TInt gotMatch;
	TUint8 *fileBuf;
	TPtr8 lineBuf(NULL,0,0);
	HBufC8 *urlBuf;
	TInt posBegin,lineSize,fileSize,len2cut=0,extraByteNL;

	LOG("Adapter::ParsePlaylistFileL - begin");

	User::LeaveIfError(file.Open(aFs, aFilename, EFileShareReadersOnly|EFileRead|EFileStreamText));
	CleanupClosePushL(file);
	User::LeaveIfError(file.Size(fileSize));

	//get the file in memory if smaller that 64k
	if ( fileSize > KMaxFileSize )
		{
		User::Leave(KErrTooBig);
		}

	fileBuf = new (ELeave) TUint8[fileSize];
	lineBuf.Set(fileBuf,fileSize,fileSize);
	file.Read(lineBuf);
	fileSize = lineBuf.Length();//should be the same as before ...
	posBegin = 0;

	while ( !done )
		{
		//construct lineBuf
		lineBuf.Set(fileBuf+posBegin,fileSize-posBegin,fileSize-posBegin);
		lineSize = lineBuf.Locate(KLineFeed); // try to locate a line feed char
		if ( lineSize == KErrNotFound )
			{
			done = ETrue;//we are at the end
			continue;
			}

		if ( lineSize > 0 && lineBuf[lineSize-1] == KCarriageReturn )
			{
			extraByteNL=1; //include also the \r
			}
		else
			{
			extraByteNL=0;
			}

		lineBuf.Set(fileBuf+posBegin,lineSize-extraByteNL,lineSize-extraByteNL);
		posBegin += (lineSize+1);

		//here we have a complete line in lineBuf, but without the \n at the end
		//parse
		if ( lineBuf.Match(_L8("File?=http://*")) == 0 )
			{
			gotMatch=1;
			len2cut = 6;
			}
		else if ( lineBuf.Match(_L8("File?*=http://*")) == 0 )
			{
			gotMatch=1;
			len2cut = 7;
			}
		else if( lineBuf.Match(_L8("Title?=*")) == 0 )
			{
			gotMatch=2;
			len2cut = 7;
			}
		else if ( lineBuf.Match(_L8("Title?*=*")) == 0 )
			{
			gotMatch=2;
			len2cut = 8;
			}
		else
			{
			gotMatch=0;
			}

		if ( gotMatch )
			{

			urlBuf = HBufC8::NewL(lineBuf.Length()-len2cut);
			*urlBuf = lineBuf.Right(lineBuf.Length()-len2cut);

			//provision for an empty title
			if( iURLs.Count()%2 && gotMatch == 1 )
			 	{
				//insert a dummy title if the previous URL did not have one
				HBufC8 *dummy = HBufC8::NewL(12);
				*dummy = _L8("- Unknown -");
				iURLs.Append(dummy);
			 	};
			if ( gotMatch != 2 || iURLs.Count()%2 )
				{
				iURLs.Append(urlBuf);//if this is a title and an even place, we skip it, because it would be a title without URL
				}
			};
		//LOG2("gotMatch %d, URL Count %d", gotMatch, iURLs.Count());
		}
	delete fileBuf;
	//close stuff now
	CleanupStack::Pop();
	file.Close();

    LOG("Adapter::ParsePlaylistFileL - end");
	}


// -----------------------------------------------------------------------------
// Exported proxy for instantiation method resolution
// Define the interface UIDs
// -----------------------------------------------------------------------------

const TImplementationProxy ImplementationTable[] =
    {
    {{KShoutcastImplementationUid}, CShoutcastAdapter::NewL}
    };


EXPORT_C const TImplementationProxy* ImplementationGroupProxy(TInt& aTableCount)
	{

    aTableCount = sizeof(ImplementationTable) / sizeof(TImplementationProxy);

	/*
	RFileLogger log;
	log.Connect();
	log.CreateLog(_L("Shoutcast"),_L("ShoutcastDLL.txt"),EFileLoggingModeOverwrite);
	log.WriteFormat(_L("Somebody is asking for me! %d"),aTableCount);
	log.CloseLog();
	log.Close();
	*/
    return ImplementationTable;
	}


TBool E32Dll(TDllReason)
    {
    return ETrue;
    }


// End of File
